/*
    myrrdin, a 2d tile engine
    Copyright (c) 2008 Friedrich Weber

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    THE SOFTWARE.
 */

module myrrdin.animatedsprite;

import dsfml.window.all;
import dsfml.system.all;
import dsfml.graphics.all;

class Frame {
    Image image;
    int length; // length in frames

    this(Image image, int length) {
	this.image = image;
	this.length = length;
    }	
}

class Animation {
    public Frame[] frames;
    private int current_frame_idx;
    private Frame current_frame;
    private int frame_counter;

    this() {
    
    }

    /* add a frame */
    void add_frame(Image image, int length) {
	this.frames ~= new Frame(image, length);
    }

    void add_frame(Frame frame) {
	this.frames ~= frame;
    }
    
    /* internal. you do not have to call it */
    void play() {
	this.current_frame = this.frames[0];
	this.current_frame_idx = 0;
    }

    /* return the current image or null if the animation ended */
    Image get_image() {
	if(this.frame_counter >= this.current_frame.length) {
	    this.frame_counter = 0;
	    this.current_frame_idx++;
	    if (this.frames.length <= this.current_frame_idx) {
		// animation stop
		return null;
	    }
	    this.current_frame = this.frames[this.current_frame_idx];
	}
	this.frame_counter++;
	return this.current_frame.image;
    }
}

class AnimatedSprite : Sprite {
    private Animation current_animation = null;
    private bool is_loop = false;

    /* play an animation now. */
    void play_animation(Animation animation, bool loop=false) {
	this.current_animation = animation; 
	this.is_loop = loop;
	this.current_animation.play();
    }

    bool is_playing() {
	return (this.current_animation !is null);
    }

    /* update everything. Perfect for an animated sprite */
    void update() {
	if(this.current_animation) {
	    Image img = this.current_animation.get_image();
	    if(img) {
		this.setImage(img);
	    }
	    else if (this.is_loop) {
		this.current_animation.play(); // is a loop, play again
	    } else {
		this.current_animation = null;
	    }
	}	    
    }	
}
